@@ -14,14 +14,22 @@ from prettytable import PrettyTable
 
 from rebotics_sdk.advanced import remote_loaders
 from rebotics_sdk.cli.common import shell, configure, roles, set_token
+from .draw import draw_annotated_drawing
 from .renderers import format_full_table
-from .utils import ReboticsCLIContext, app_dir, pass_rebotics_context, read_saved_role, process_role
+from .utils import ReboticsCLIContext, app_dir, pass_rebotics_context, read_saved_role, process_role, \
+    downloads_with_threads, save_masks, get_segmentation, refresh_url, get_segmentation_mode
 from ..advanced.flows import FileUploadError
 from ..advanced.packers import ClassificationDatabasePacker, ClassificationDatabaseException, \
     DuplicateFeatureVectorsException
 from ..providers import AdminProvider, RetailerProvider
 from ..utils import uri_validator, get_filename_from_url
 
+OPENCV_INSTALLED = True
+try:
+    import cv2
+except ModuleNotFoundError:
+    OPENCV_INSTALLED = False
+
 
 @click.group()
 @click.option('-f', '--format', default='table', type=click.Choice(['table', 'id', 'json']), help='Result rendering')
@@ -434,15 +442,20 @@ def stitching():
 @click.argument("argument", type=click.STRING)
 @click.option('-o', "--output", type=click.Path(dir_okay=True, file_okay=False))
 @click.option('-c', '--concurrency', default=8, type=click.INT)
+@click.option('-d', '--draw', is_flag=True, help='To draw annotated frames')
 @pass_rebotics_context
-def stitching_setup(ctx, argument, concurrency, output):
+def stitching_setup(ctx, argument, concurrency, output, draw):
     """
     argument - id of stitching debug data from admin, or json file
     """
+    if draw and not OPENCV_INSTALLED:
+        raise click.ClickException(' OpenCv not found, but can be installed with: pip install opencv-python ')
+
     if output:
         output = pathlib.Path(output).absolute()
     else:
         output = pathlib.Path('.').absolute()
+
     output = output / f"stitching_{argument.replace(' ', '_')}"
     output.mkdir(parents=True, exist_ok=True)
     ctx.verbose_log(f"Target directory is: {output}")
@@ -477,6 +490,14 @@ def stitching_setup(ctx, argument, concurrency, output):
 
     files_to_load = []  # pair of remote_url to local path
 
+    segmentation_filepath = output / "image_segmentation_debug.json"
+    segmentation_url = definition.get("image_segmentation_debug", None)
+    if segmentation_url is not None:
+        files_to_load.append([segmentation_url, segmentation_filepath])
+    else:
+        draw = False
+        ctx.verbose_log("Cannot draw bboxes, no image segmentation debug file is available")
+
     for call_count, call_data in enumerate(definition['calls']):
         call_folder = output / f"call_{call_count}"
         call_folder.mkdir(parents=True, exist_ok=True)
@@ -491,7 +512,9 @@ def stitching_setup(ctx, argument, concurrency, output):
         }
 
         del stitching_input['frame_urls']
-        del stitching_input['annotated_frames']
+        # for backward compatible
+        if 'annotated_frames' in stitching_input:
+            del stitching_input['annotated_frames']
 
         # add stitching result image
         try:
@@ -536,15 +559,23 @@ def stitching_setup(ctx, argument, concurrency, output):
 
     ctx.verbose_log(f"Registered files for downloading: {len(files_to_load)}")
 
-    with Pool(concurrency) as pool:
-        for _ in tqdm.tqdm(
-            pool.starmap(remote_loaders.download, files_to_load),
-            desc=f"Downloading files to {str(output)}",
-            total=len(files_to_load),
-        ):
-            pass
-        pool.close()
-        pool.join()
+    downloads_with_threads(files_to_load, concurrency=len(files_to_load))
+    click.echo(f"Stitching setup complete in: {output}")
+
+    if draw:
+        segmentation_per_image = get_segmentation(segmentation_filepath)
+        mode = get_segmentation_mode(segmentation_per_image)
+        if mode == 'remote_url':
+            refreshed_urls = refresh_url(ctx.provider.refresh_url, segmentation_per_image)
+            # save masks in output/all_masks
+            save_masks(output, refreshed_urls)
+
+        draw_annotated_drawing(
+            root_folder=output,
+            segmentation_per_image=segmentation_per_image,
+            mode=mode,
+            concurrency=concurrency
+       )
 
 
 api.add_command(shell, 'shell')
