@@ -1,12 +1,16 @@
 import json
 import os
+from concurrent.futures import ThreadPoolExecutor, as_completed
 from multiprocessing import Pool
+from urllib.parse import urlparse, unquote
 
 import click
 import six
+import tqdm
 
 from rebotics_sdk.cli.renderers import format_processing_action_output, format_full_table
 from rebotics_sdk.utils import download_file
+from ..advanced import remote_loaders
 from ..providers import RetailerProvider
 from ..utils import mkdir_p
 
@@ -187,3 +191,72 @@ def guess_input_type(ext):
         return 'video'
     else:
         raise UnrecognizedInputTypeByExtension('File with extension %s is given' % ext)
+
+
+def downloads_with_threads(files, concurrency):
+    results = []
+    with ThreadPoolExecutor(concurrency) as executor:
+        futures = [
+            executor.submit(remote_loaders.download, file[0], file[1])
+            for file in files
+        ]
+        with click.progressbar(length=len(futures), label='Downloading files:') as bar:
+            for future in as_completed(futures):
+                results.append(future.result())
+                bar.update(1)
+    return results
+
+
+def refresh_with_threads(refresh_url, per_image, concurrency):
+    results = []
+    with ThreadPoolExecutor(concurrency) as executor:
+        futures = [
+            executor.submit(refresh_url, img_dict['remote_url'])
+            for img_dict in per_image
+        ]
+        with click.progressbar(length=len(futures), label='Refreshing urls:') as bar:
+            for future in as_completed(futures):
+                results.append(future.result()['url'])
+                bar.update(1)
+    return results
+
+
+def run_with_processes(invoked, iterable, concurrency):
+    with Pool(concurrency) as pool:
+        for _ in tqdm.tqdm(
+            pool.starmap(invoked, iterable)
+        ):
+            pass
+        pool.close()
+        pool.join()
+
+
+def get_segmentation(segmentation_filepath):
+    with open(segmentation_filepath, 'r') as fd:
+        segmentation_file = json.load(fd)
+    return segmentation_file['per_image']
+
+
+def get_segmentation_mode(segmentation):
+    mode = 'items'
+    if segmentation[0].get('remote_url') is not None:
+        mode = 'remote_url'
+    return mode
+
+
+def refresh_url(refresh_url, segmentation_per_image):
+    concurrency = len(segmentation_per_image)
+    refreshed_urls = refresh_with_threads(refresh_url, segmentation_per_image, concurrency)
+    return refreshed_urls
+
+
+def save_masks(root_folder, urls):
+        path_to_save_masks = root_folder / 'all_masks'
+        path_to_save_masks.mkdir(parents=True, exist_ok=True)
+
+        list_with_downloaded_mask = [
+            [url, path_to_save_masks / str(urlparse(unquote(url)).path).lstrip('/').split('/')[-1]]
+            for url in urls
+        ]
+
+        downloads_with_threads(list_with_downloaded_mask, concurrency=len(list_with_downloaded_mask))
